/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 7.11, Version 1.0, 2001-04-24
****************************************************/

-- Define a CHECK constraint in a single column
-- using the default constraint name
-- specifying NOT FOR REPLICATION

CREATE TABLE NewEmployees (
	EmployeeID int NOT NULL
	, EmployeeName varchar(50) NOT NULL
	, PostCode char(5) NOT NULL
		CHECK NOT FOR REPLICATION 
			(PostCode LIKE '[0-9][0-9][0-9][0-9][0-9]')
)
GO

DROP TABLE NewEmployees
GO

-- Define multiple CHECK constraint
-- in existing tables specifying
-- the constraint name and defining
-- the constraint at table level
-- using the ALTER TABLE statement
-- checking existing data
-- specifying NOT FOR REPLICATION

CREATE TABLE NewOrders (
	OrderID int NOT NULL
	, CustomerID int NOT NULL
	, SaleDate smalldatetime NOT NULL 
	, DueDate smalldatetime NOT NULL
	, ShipmentMethod char(1) NOT NULL
)

ALTER TABLE NewOrders
ADD CONSTRAINT CC_Order_DueDate
	CHECK NOT FOR REPLICATION
		(DATEDIFF(day, SaleDate, DueDate) <= 90)

ALTER TABLE NewOrders
ADD CONSTRAINT CC_Order_SaleDate
	CHECK NOT FOR REPLICATION
		(DATEDIFF(day, CURRENT_TIMESTAMP, SaleDate) <= 0)

ALTER TABLE NewOrders
ADD CONSTRAINT CC_Order_Shipment
	CHECK NOT FOR REPLICATION
		(ShipmentMethod IN ('A', 'L', 'S'))

GO

DROP TABLE NewOrders
GO
