/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 7.19, Version 1.0, 2001-04-24
****************************************************/

-- Create a DEFAULT object using a constant

CREATE DEFAULT NoSales
	AS 0
GO

-- Create DEFAULT objects using expressions
-- based on built-in functions

CREATE DEFAULT ThisMonth
	AS Month(CURRENT_TIMESTAMP)
GO

CREATE DEFAULT UserDB
	AS SYSTEM_USER
		+ ' - ' + DB_NAME(DB_ID())
GO

-- Create two User-Defined Data Types

EXEC sp_addtype 'UDDTLoginDB', 'nvarchar(256)', 'NULL'

EXEC sp_addtype 'UDDTSales', 'money', 'NULL'
GO

-- Create a table to test the DEFAULT objects
-- and the User-Defined Data Types

CREATE TABLE TestDefaults(
	  ID int NOT NULL
		IDENTITY(1,1)
		PRIMARY KEY
	, TotalSales money NULL
	, SalesMonth tinyint NULL
	, WhoWhere UDDTLoginDB
)
GO

-- Insert a new empty row in the table

INSERT TestDefaults
DEFAULT VALUES

PRINT char(10) + 'No defaults defined' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Bind the NoSales DEFAULT object
-- to the TotalSales field in the TestDefaults table

EXEC sp_bindefault 'NoSales', 'TestDefaults.TotalSales'
GO

-- Insert a new empty row in the table

INSERT TestDefaults
DEFAULT VALUES

PRINT  CHAR(10) + 'Only DEFAULT on TotalSales defined' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Bind the ThisMonth DEFAULT object
-- to the SalesMonth field in the TestDefaults table

EXEC sp_bindefault 'ThisMonth', 'TestDefaults.SalesMonth'

GO

-- Insert a new empty row in the table

INSERT TestDefaults
DEFAULT VALUES

PRINT  CHAR(10) + 'DEFAULT defined on TotalSales and SalesMonth' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Bind the UserDB DEFAULT object
-- to the UDDTLoginDB User-Defined Data Type

EXEC sp_bindefault 'UserDB', 'UDDTLoginDB'

GO

-- Insert a new empty row in the table

INSERT TestDefaults
DEFAULT VALUES

PRINT  CHAR(10) + 'DEFAULT defined on TotalSales, SalesMonth'
PRINT 'and the UDDTLoginDB User-Defined Data Type' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Add a new column to the TestDefaults table
-- Using the UDDTSales data type

ALTER TABLE TestDefaults
ADD ProjectedSales UDDTSales
GO

PRINT CHAR(10) + 'Add an empty field using the UDDTSales data type' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Bind the NoSales DEFAULT object
-- to the UDDTSales User-Defined Data Type
-- for future columns only

EXEC sp_bindefault 'NoSales', 'UDDTSales', 'futureonly'
GO

-- Insert a new empty row in the table

INSERT TestDefaults
DEFAULT VALUES

PRINT CHAR(10) + 'DEFAULT defined on UDDTSales data type as futureonly'
PRINT 'does not affect the existing fields using this UDDT' + CHAR(10)

SELECT *
FROM TestDefaults
GO

-- Drop everything in order
-- Table first
-- UDDT next
-- DEFAULT last

DROP TABLE TestDefaults

EXEC sp_droptype 'UDDTSales'

EXEC sp_droptype 'UDDTLoginDB'

DROP DEFAULT NoSales

DROP DEFAULT ThisMonth

DROP DEFAULT UserDB
