/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.2, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Add the DriverNo fields to the Employees table

ALTER TABLE Employees
ADD DriverNo varchar(15) NULL
GO

-- Create the trigger to check
-- for uniqueness of the DriverNo field 

CREATE TRIGGER isrEmployees
ON Employees
FOR INSERT, UPDATE
AS

	IF UPDATE (DriverNo)

		IF EXISTS (
			SELECT DriverNo, COUNT(*)
			FROM Employees
			WHERE DriverNo IS NOT NULL
				AND DriverNo IN (
					SELECT DISTINCT DriverNo
					FROM Inserted
					)
			GROUP BY DriverNo
			HAVING COUNT(*) > 1
			)
		BEGIN
			RAISERROR ('Driver license number not unique, INSERT aborted', 16, 1)
			ROLLBACK TRAN
		END
GO

-- This statement succeeds

UPDATE Employees
SET DriverNo = '74914173'
WHERE EmployeeID = 5

-- This statement fails
-- the driverno is not unique

UPDATE Employees
SET DriverNo = '74914173'
WHERE EmployeeID = 6

-- This statement fails
-- Trying to insert multiple repeated
-- DriverNo values

UPDATE Employees
SET DriverNo = '74914175'
WHERE EmployeeID BETWEEN 6 AND 10

-- reset to NULL succeeds because
-- NULL values are not considered
-- in this trigger

UPDATE Employees
SET DriverNo = NULL
