/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.9, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Create a new table

CREATE TABLE NewCategories (
	  CategoryID int NOT NULL
		IDENTITY(1,1) 
		PRIMARY KEY
	, CategoryName nvarchar(15) NOT NULL 
	, Modified smalldatetime
		DEFAULT CURRENT_TIMESTAMP
	, Modifier nchar(128)
		DEFAULT SYSTEM_USER
)
GO

-- Create an AFTER UPDATE trigger

CREATE TRIGGER tr_NewCategories
ON NewCategories
AFTER UPDATE, INSERT, DELETE
AS

	DECLARE @s varchar(80)
	DECLARE @ni int, @nd int

	SELECT @ni = COUNT(*)
	FROM Inserted
	
	SELECT @nd = COUNT(*)
	FROM Deleted
	
	SET @s = CASE
		WHEN (@ni = @nd) AND @ni > 0
			THEN 'AFTER UPDATE Trigger (compare Deleted and Inserted tables)'
		WHEN @ni > @nd
			THEN 'AFTER INSERT Trigger (look at the Inserted table)'
		WHEN @ni < @nd
			THEN 'AFTER DELETE Trigger (look at the Deleted table)'
		ELSE 'TRIGGER fired by Null action (Inserted and Deleted are empty)' END
	
	PRINT CHAR(10) + '#######################################################'
	PRINT @s
	PRINT '#######################################################' + CHAR(10)
	
	PRINT CHAR(10) + 'Inserted Table' + CHAR(10)
	
	SELECT *
	FROM Inserted
	
	PRINT CHAR(10) + 'Deleted Table' + CHAR(10)
	
	SELECT *
	FROM Deleted
GO

-- Insert Operation

INSERT NewCategories (CategoryName)
VALUES ('Food')

-- Multiple Insert Operation

INSERT NewCategories (CategoryName)
SELECT CategoryName
FROM Categories

-- Delete one row

DELETE NewCategories
WHERE CategoryID = 7

-- Delete multiple rows

DELETE NewCategories
WHERE CategoryID BETWEEN 1 AND 4

-- Update one row

UPDATE NewCategories
SET CategoryName = 'Products'
WHERE CategoryID = 8

-- Update multiple rows

UPDATE NewCategories
SET CategoryName = UPPER(CategoryName)

-- Null operation because it does not affect any row

UPDATE NewCategories
SET CategoryName = UPPER(CategoryName)
WHERE CategoryID > 1000

-- Drop the demo table
-- it drops automatically the trigger

DROP TABLE NewCategories
