/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.10, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Create a view based in the Employees table

CREATE VIEW EmployeesName
AS
	SELECT ISNULL(EmployeeID, 0) AS EmployeeID
		, LastName + ', ' + FirstName AS FullName
	FROM Employees
GO

-- Show information related to the Employee 6

SELECT FullName
FROM EmployeesName
WHERE EmployeeID = 6

SELECT FirstName
	, LastName
FROM Employees
WHERE EmployeeID = 6
GO

-- Create INSTEAD OF INSERT trigger

CREATE TRIGGER isrEmployeesName
ON EmployeesName
INSTEAD OF INSERT
AS
	INSERT Employees (LastName, FirstName)
	SELECT LEFT(FullName, CHARINDEX(',', FullName) - 1)
		, LTRIM(RIGHT(FullName, LEN(FullName)
			- CHARINDEX(',', FullName)))
	FROM Inserted
GO

-- Create INSTEAD OF UPDATE trigger

CREATE TRIGGER udtEmployeesName
ON EmployeesName
INSTEAD OF UPDATE
AS
	UPDATE Employees
	SET LastName = LEFT(Inserted.FullName
			, CHARINDEX(',', Inserted.FullName) - 1)
		, FirstName = LTRIM(RIGHT(Inserted.FullName,
			LEN(Inserted.FullName)
			- CHARINDEX(',', Inserted.FullName) ))
	FROM Employees
		JOIN Inserted
			ON Inserted.EmployeeID = Employees.EmployeeID
	WHERE Employees.EmployeeID = Inserted.EmployeeID
GO

-- testing INSTEAD OF UPDATE Trigger

UPDATE EmployeesName
SET FullName = 'NewFamily, Michael'
WHERE EmployeeID = 6
GO

SELECT FullName
FROM EmployeesName
WHERE EmployeeID = 6

SELECT FirstName, LastName
FROM Employees
WHERE EmployeeID = 6
GO

-- testing INSTEAD OF INSERT Trigger

INSERT EmployeesName (EmployeeID, FullName)
VALUES (0, 'NewFamily, NewGuy')
GO

SELECT FullName
FROM EmployeesName
WHERE FullName LIKE 'New%'

SELECT FirstName, LastName
FROM Employees
WHERE LastName LIKE 'New%'
GO

-- deleting from this view
-- does not require an INSTEAD OF DELETE Trigger

DELETE EmployeesName
WHERE FullName = 'NewFamily, NewGuy'
GO

SELECT FullName
FROM EmployeesName
WHERE FullName LIKE 'New%'

SELECT FirstName, LastName
FROM Employees
WHERE LastName LIKE 'New%'
GO

DROP VIEW EmployeesName
GO
