/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 9.11, Version 1.0, 2001-04-24
****************************************************/

USE Northwind
GO

-- Modify the Categories table
-- To add a new field to store
-- Budget value

ALTER TABLE Categories
ADD Budget money NULL
GO
-- Add a random budget per category

UPDATE Categories
SET Budget = ROUND(SIN(CategoryID)* 10000 + 20000, -3)
GO

-- Create a view to see the total budget

CREATE VIEW TotalBudget
AS
	SELECT SUM(Budget) AS TBudget
	FROM Categories
GO

-- Create an INSTEAD OF trigger
-- to UPDATE budgets proportionally

CREATE TRIGGER udtTotalBudget
ON TotalBudget
INSTEAD OF UPDATE
AS

	-- Share the new budget proportionally

	UPDATE Categories
	SET Budget = ROUND(Budget *
			(Inserted.TBudget /
			Deleted.TBudget), -3)
	FROM Inserted, Deleted
	
	-- Adjust the difference to the biggest budget
	
	UPDATE Categories
	SET Budget = Budget +
			Inserted.TBudget -
			(SELECT SUM(Budget)
			FROM Categories)
	FROM Inserted
	WHERE Budget =
		(SELECT MAX(Budget)
		FROM categories)
	
GO

-- Test the view

PRINT CHAR(10) + 'Total Budget before the update' + CHAR(10)

SELECT *
FROM TotalBudget

PRINT CHAR(10) + 'Individual Budgets before the update' + CHAR(10)

SELECT CategoryID, Budget
FROM Categories

-- Increase the budget

UPDATE TotalBudget
SET TBudget = 200000

-- Test the update

PRINT CHAR(10) + 'Total Budget after the update' + CHAR(10)

SELECT *
FROM TotalBudget

PRINT CHAR(10) + 'Individual Budgets after the update' + CHAR(10)

SELECT CategoryID, Budget
FROM Categories

-- Remove the view and the Budget column

DROP VIEW TotalBudget

ALTER TABLE Categories
DROP COLUMN Budget
