/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 10.4, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

-- Returns the maximum ProductID from Products

CREATE FUNCTION dbo.MaxProductID
()
RETURNS int
AS
BEGIN
	RETURN (
		SELECT MAX(ProductID)
		FROM dbo.Products
	)
END
GO

-- Returns who and from where the query is executed

CREATE FUNCTION dbo.WhoWhere
()
RETURNS nvarchar(256)
AS
BEGIN
	RETURN SYSTEM_USER
		+ ' FROM '
		+ APP_NAME()
END
GO

-- Returns the date of the latest executed statement
-- which is usually today

CREATE FUNCTION dbo.Today
()
RETURNS smalldatetime
AS
BEGIN
	DECLARE @sdt smalldatetime
	
	SELECT @SDT = CONVERT(varchar(10), MAX(last_batch), 112)
	FROM master.dbo.sysprocesses
	
	RETURN @SDT
	
END
GO

-- Function that produces a non-predictable
-- Pseudo-Random Number

CREATE FUNCTION dbo.PRand
()
RETURNS float
AS
BEGIN
	DECLARE @dt datetime
	DECLARE @dts varchar(3)
	DECLARE @t1 float
	DECLARE @t2 float
	DECLARE @r float
	
	-- Obtain the time of latest executed statement
	
	SET @dt = (
		SELECT MAX(last_batch)
		FROM master.dbo.sysprocesses
		)
	
	-- Select only the milliseconds
	
	SET @dts = RIGHT(CONVERT(varchar(20), @dt, 114) , 3)
	
	-- Scramble the digits
	
	SET @t1 = CAST(SUBSTRING(@dts, 2, 1)
		+ RIGHT(@dts, 1)
		+ LEFT(@dts, 1) AS int)
	
	-- Obtain the time of latest executed statement
	
	SET @dt = (
		SELECT MAX(last_batch)
		FROM master.dbo.sysprocesses
		)
	
	-- Select only the milliseconds
	
	SET @dts = RIGHT(CONVERT(varchar(20), @dt, 114) , 3)
	
	-- Scramble the digits
	
	SET @t2 = CAST(SUBSTRING(@dts, 2, 1)
		+ RIGHT(@dts, 1)
		+ LEFT(@dts, 1) AS int)
	
	-- Select the random number
	
	SET @r = '0' + LEFT(RIGHT(CONVERT(varchar(40), @t1 * @t2 / pi(), 2), 21), 16)
	
	-- Return the random number
	
	RETURN @r
END
GO
