/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation
** Source code. Listing 10.7, Version 1.0, 2001-04-25
****************************************************/

USE Northwind
GO

-----------------------------------------------------------
-- Generic function to compute the total price of a sale
-- from the quantity, unitprice and discount
-----------------------------------------------------------

CREATE FUNCTION dbo.TotalPrice
(@Quantity float, @UnitPrice money, @Discount float = 0.0)
RETURNS money
AS
BEGIN
	RETURN (@Quantity * @UnitPrice * (1.0 - @Discount))
END
GO

-----------------------------------------------------------
-- Compute the future value of an annuity based on
-- periodic fixed payments with a fixed interest rate
-- Parameters:
-- @rate: interest rate between payments
-- @nper: number of payments
-- @pmt: payment to be made on every period
-- @pv: present value. Default to 0.0
-- @type: 0 if the payment is made at the end of each period (default)
--        1 if the payment is made at the beginning of each period
-----------------------------------------------------------

CREATE FUNCTION dbo.fn_FV
(@rate float, @nper int, @pmt money, @pv money = 0.0, @type bit = 0)
RETURNS money
AS
BEGIN
	DECLARE @fv money
	
	IF @rate = 0
		SET @fv = @pv + @pmt * @nper
	ELSE
		SET @fv = @pv * POWER(1 + @rate, @nper) 
			+ @pmt * (((POWER(1 + @rate, @nper + @type) - 1) / @rate) - @type)
	
	RETURN (-@fv)
END
GO


-----------------------------------------------------------
-- Encrypt the string increasing the Unicode value of every
-- character by the number of characters in the string
-----------------------------------------------------------

CREATE Function dbo.SimpleEncrypt
(@string nvarchar(4000))
RETURNS nvarchar(4000)
AS
BEGIN
	DECLARE @output nvarchar(4000)
	DECLARE @i int, @l int, @c int
	
	SET @i = 1
	SET @l = len(@string)
	SET @output = ''
	
	WHILE @i <= @l
	BEGIN
	
		SET @c = UNICODE(SUBSTRING(@string, @i, 1))
		
		SET @output = @output 
			+ 	CASE
					WHEN @c > 65535 - @l
						THEN NCHAR(@c + @l - 65536)
					ELSE NCHAR(@c + @l) 
				END
		
		SET @i = @i + 1
	
	END
	
	RETURN @output
	
END
GO


-----------------------------------------------------------
-- Decrypt the string decreasing the Unicode value of every
-- character by the number of characters in the string
-----------------------------------------------------------

CREATE Function dbo.SimpleDecrypt
(@string nvarchar(4000))
RETURNS nvarchar(4000)
AS
BEGIN
	DECLARE @output nvarchar(4000)
	DECLARE @i int, @l int, @c int
	
	SET @i = 1
	SET @l = len(@string)
	SET @output = ''
	
	WHILE @i <= @l
	BEGIN
	
		SET @c = UNICODE(SUBSTRING(@string, @i, 1))
		
		SET @output = @output 
			+ 	CASE
					WHEN @c - @l >= 0
						THEN NCHAR(@c - @l)
					ELSE NCHAR(@c + 65535 - @l) 
				END
		
		SET @i = @i + 1
	
	END
	
	RETURN @output
	
END
GO 
