/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 10.18, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

CREATE FUNCTION dbo.tv_GetCustomersFromCountry
	(@country nvarchar(15))
RETURNS @List TABLE
	(CustomerID nchar(5)
	, CompanyName nvarchar(40)
	, Country nvarchar(15))
AS
BEGIN

	INSERT @List
		SELECT CustomerID, CompanyName, Country
		FROM Customers
		WHERE Country = @country
	
	RETURN
END
GO

-- Returns USA-based customers

CREATE FUNCTION dbo.tv_GetCustomersFromUSA
()
RETURNS @List TABLE
	(CustomerID nchar(5)
	, CompanyName nvarchar(40)
	, Country nvarchar(15))
AS
BEGIN

	INSERT @List
		SELECT CustomerID, CompanyName, Country
		FROM dbo.GetCustomersFromCountry(N'USA')
	
	RETURN
END
GO

-- Returns the orders from a specific day

CREATE FUNCTION dbo.tv_GetOrdersFromDay
(@date as smalldatetime)
RETURNS @list TABLE
	(OrderID int
	, OrderDate datetime)
AS
BEGIN

	INSERT @List
		SELECT OrderID, OrderDate
		FROM Orders
		WHERE DATEDIFF(day, OrderDate, @date) = 0
	
	RETURN
END
GO

-- Returns orders from today

CREATE FUNCTION dbo.tv_GetOrdersFromToday
()
RETURNS @list TABLE
	(OrderID int
	, OrderDate datetime)
AS
BEGIN

	INSERT @List
		SELECT OrderID, OrderDate
		FROM Orders
		WHERE DATEDIFF(day, OrderDate, dbo.Today()) = 0
	
	RETURN
END
GO

-- Returns Orders with the total order value

CREATE FUNCTION dbo.tv_OrdersWithValue
()
RETURNS @list TABLE
	(OrderID int
	, CustomerID nchar(5)
	, OrderDate datetime
	, TotalValue money)
AS
BEGIN

	INSERT @List
		SELECT O.OrderID, CustomerID, OrderDate, TotalValue
		FROM Orders O
		JOIN (
			SELECT OrderID
				, SUM(dbo.TotalPrice(Quantity, UnitPrice, Discount))
			AS TotalValue
			FROM [Order Details]
			GROUP BY OrderID
			) AS OD
			ON O.OrderID = OD.OrderID
	
	RETURN
END
GO

-- Returns orders with a value greater than
-- a specific target value

CREATE FUNCTION dbo.tv_OrdersByValue
(@total money)
RETURNS @list TABLE
	(OrderID int
	, CustomerID nchar(5)
	, OrderDate datetime
	, TotalValue money)
AS
BEGIN

	INSERT @List
		SELECT OrderID, CustomerID, OrderDate, TotalValue
		FROM dbo.OrdersWithValue()
		WHERE TotalValue > @total
	
	RETURN
END
GO

-- Returns the top 10 orders by total value

CREATE FUNCTION dbo.tv_TopTenOrders
()
RETURNS @list TABLE
	(OrderID int
	, CustomerID nchar(5)
	, OrderDate datetime
	, TotalValue money)
AS
BEGIN

	INSERT @List
		SELECT TOP 10 WITH TIES
			OrderID, CustomerID, OrderDate, TotalValue
		FROM dbo.OrdersWithValue()
		ORDER BY TotalValue DESC
	
	RETURN
END
GO
