/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 10.19, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

-- Converts a string containing a list of items
-- into a single column table where every item
-- is in a separate row
-- using any character as a separator

CREATE FUNCTION dbo.MakeList
	(@ParamArray as nvarchar(4000)
	, @Separator as char(1) = '|')
RETURNS @List TABLE
	(Item sql_variant)
AS
BEGIN

	DECLARE @pos int, @pos0 int
	
	SET @pos0 = 0
	
	WHILE 1=1
	BEGIN
	
		SET @pos = CHARINDEX(@Separator, @ParamArray, @pos0 + 1)
		
		INSERT @List
			SELECT CASE @pos
				WHEN 0 THEN SUBSTRING(@ParamArray, @pos0+1, LEN(@ParamArray) - @pos -1)
				ELSE SUBSTRING(@ParamArray, @pos0+1, @pos - @pos0-1)
				END
		
		IF @pos = 0 BREAK
		
		SET @pos0 = @pos
	
	END
	
	RETURN
	
END
GO

-- Produces a list of orders
-- with full descriptive information
-- ProductName, CategoryName, CompanyName
-- OrderDate and TotalValue
-- with every primary key to link to other tables
-- The list can be produced for every
-- Order (@Key = 'ORD'),
-- Product (@Key = 'PRO'),
-- Customer (@Key = 'CUS'),
-- Category (@Key = 'CAT')
-- Full List (@Key NOT IN ('ORD', 'PRO', 'CUS', 'CAT'))

CREATE FUNCTION dbo.OrderDetailsComplete
	(@ID sql_variant = NULL,
	@Key char(3) = NULL)
RETURNS @Details TABLE
	(OrderID int
	, ProductID int
	, CustomerID nchar(5) NULL
	, CategoryID int NULL
	, OrderDate smalldatetime NULL
	, Value money NULL
	, Category nvarchar(15) NULL
	, Product nvarchar(40) NULL
	, Company nvarchar(40) NULL)
AS
BEGIN

	IF @Key = 'ORD'
	BEGIN
	
		INSERT @Details
			(OrderID, ProductID, Value)
			SELECT OrderID, ProductID,
				dbo.TotalPrice(Quantity, UnitPrice, Discount)
			FROM [Order Details]
			WHERE OrderID = @ID
		
	END
	ELSE IF @Key = 'PRO'
	BEGIN
	
		INSERT @Details 
			(OrderID, ProductID, Value)
			SELECT OrderID, ProductID,
				dbo.TotalPrice(Quantity, UnitPrice, Discount)
			FROM [Order Details]
			WHERE ProductID = @ID
		
	END
	ELSE IF @Key = 'CUS'
	BEGIN
	
		INSERT @Details 
			(OrderID, ProductID, CustomerID, Value)
			SELECT O.OrderID, ProductID, CustomerID,
				dbo.TotalPrice(Quantity, UnitPrice, Discount)
			FROM [Order Details] OD
				JOIN Orders O
					ON O.OrderID = OD.OrderID
			WHERE CustomerID = @ID
		
	END
	ELSE IF @Key = 'CAT'
	BEGIN
	
		INSERT @Details 
			(OrderID, ProductID, CategoryID, Value)
			SELECT OD.OrderID, P.ProductID, CategoryID,
				dbo.TotalPrice(Quantity, OD.UnitPrice, Discount)
			FROM [Order Details] OD
				JOIN Products P
					ON P.ProductID = OD.ProductID
			WHERE CategoryID = @ID
		
	END
	ELSE
	BEGIN
	
		INSERT @Details
			(OrderID, ProductID, Value)
		SELECT OrderID, ProductID,
			dbo.TotalPrice(Quantity, UnitPrice, Discount)
		FROM [Order Details]
		
	END
	
	UPDATE D
	SET D.CustomerID = O.CustomerID
		, D.OrderDate = O.OrderDate
	FROM @Details D
		JOIN Orders O
			ON O.OrderID = D.OrderID
	WHERE D.CustomerID IS NULL
	
	UPDATE D
	SET D.CategoryID = P.CategoryID
		, D.Product = P.ProductName
	FROM @Details D
		JOIN Products P
			ON P.ProductID = D.ProductID
	WHERE D.CategoryID IS NULL
	
	UPDATE D
	SET D.Category = C.CategoryName
	FROM @Details D
		JOIN Categories C
			ON C.CategoryID = D.CategoryID
	
	UPDATE D
	SET D.Company = C.CompanyName
	FROM @Details D
		JOIN Customers C
			ON C.CustomerID = D.CustomerID
	
	RETURN
	
END
GO
