/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 11.11, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

SET NOCOUNT ON
GO

-- Retrieve the best selling product by number of units and by revenue

-- This solution uses subqueries

	SELECT 'By units' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products
	WHERE ProductID = 
		(
			SELECT ProductID
			FROM [Order Details]
			GROUP BY ProductID
			HAVING SUM(Quantity) = 
				(
					SELECT MAX(SQ)
					FROM 
						(
							SELECT SUM(Quantity) as SQ
							FROM [Order Details]
							GROUP BY ProductID
						) AS OD
				)
		)

UNION

	SELECT 'By revenue' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products
	WHERE ProductID = 
		(
			SELECT ProductID
			FROM [Order Details]
			GROUP BY ProductID
			HAVING SUM(UnitPrice * Quantity * (1-Discount)) = 
				(
					SELECT MAX(SQ)
					FROM 
						(
							SELECT SUM(UnitPrice * Quantity * (1-Discount)) as SQ
							FROM [Order Details]
							GROUP BY ProductID
						) AS OD
				)
		)

-- This solution uses subqueries as well

	SELECT 'By units' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products P
		JOIN 
			(
				SELECT TOP 1 ProductID
					, SUM(Quantity) AS SQ
				FROM [Order Details]
				GROUP BY productID
				ORDER BY SQ DESC
			) AS OD
	ON OD.ProductID = P.ProductID
	
UNION

	SELECT 'By revenue' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products P
		JOIN 
			(
				SELECT TOP 1 ProductID
					, SUM(UnitPrice * Quantity * (1-Discount)) AS SR
				FROM [Order Details]
				GROUP BY ProductID
				ORDER BY SR DESC
			) AS OD
	ON OD.ProductID = P.ProductID

-- This solution does not use subqueries. 
-- However the execution is similar to the query that uses subqueries

SELECT ProductID
	, SUM(Quantity) AS SQ
	, CAST(SUM(UnitPrice * Quantity * (1.0-Discount))AS money) as SR
INTO #BestSelling
FROM [Order Details]
WHERE ProductID IS NOT NULL
GROUP BY productID

DECLARE @MQ int, @MR money
DECLARE @PQ int, @PR int

SELECT @MQ = MAX(SQ)
	, @MR = MAX(SR)
FROM #BestSelling

SELECT @PQ = ProductID
FROM #BestSelling
WHERE SQ = @MQ

SELECT @PR = ProductID
FROM #BestSelling
WHERE SR = @MR

-- Start the UNION query

	SELECT 'By units' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products
	WHERE ProductID = @PQ

UNION

	SELECT 'By revenue' AS Criteria
		, ProductName as 'Best Selling'
	FROM Products
	WHERE ProductID = @PR

-- UNION query terminates

-- drop temporary table

DROP TABLE #BestSelling
