/***************************************************
** Microsoft SQL Server 2000 Programming by Example
** ISBN 0-7897-2449-9
** Carlos Eduardo Rojas and Fernando G. Guerrero
** Copyright 2001 by QUE Corporation,
** Source code. Listing 13.1, Version 1.0, 2001-04-04
****************************************************/


USE Northwind
GO

SET NOCOUNT ON
GO

-----------------------
-- Without Transactions
-----------------------

DECLARE @CatID int, @ProdID int

-- Operation 1
-- Create a new Category

INSERT Categories
	(CategoryName)
VALUES ('Cars')

-- Retrieves the latest IDENTITY value inserted

SET @CatID = SCOPE_IDENTITY()

-- Operation 2
-- Create a new product
-- in the new Category

INSERT Products
	(ProductName, CategoryID)
VALUES ('BigCars', @CatID)

-- Retrieves the latest IDENTITY value inserted

SET @ProdID = SCOPE_IDENTITY()

-- Operation 3
-- Change UnitsInStock
-- for the new product

UPDATE Products
SET UnitsInStock = 20
WHERE ProductID = @ProdID

-----------------------
-- With Transactions
-----------------------

-- Start a new transaction

BEGIN TRAN

	-- Operation 4
	-- Create a new Category
	
	INSERT Categories
		(CategoryName)
	VALUES ('HiFi')
	
	IF @@ERROR <> 0 GOTO AbortTransaction
	
	SELECT @CatID = CategoryID
	FROM Categories
	WHERE CategoryName = 'HiFi'
	
	-- Operation 2
	-- Create a new product
	-- in the new Category
	
	INSERT Products
		(ProductName, CategoryID)
	VALUES ('GreatSound', @CatID)
	
	IF @@ERROR <> 0 GOTO AbortTransaction
	
	SELECT @ProdID = ProductID
	FROM Products
	WHERE ProductName = 'GreatSound'
	
	-- Operation 3
	-- Change UnitsInStock
	-- for the new product
	
	UPDATE Products
	SET UnitsInStock = 50
	WHERE ProductID = @ProdID
	
	IF @@ERROR <> 0 GOTO AbortTransaction

COMMIT TRAN
PRINT 'Transaction committed'

GOTO EndTransaction

AbortTransaction:

	ROLLBACK TRAN
	PRINT 'Transaction rolled back'

EndTransaction:

	PRINT 'Transaction finished'
